@@ -17,8 +17,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -30,7 +33,7 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathySearchBar)
 
-G_DEFINE_TYPE (EmpathySearchBar, empathy_search_bar, GTK_TYPE_BIN);
+G_DEFINE_TYPE (EmpathySearchBar, empathy_search_bar, GTK_TYPE_BOX);
 
 typedef struct _EmpathySearchBarPriv EmpathySearchBarPriv;
 struct _EmpathySearchBarPriv
@@ -41,6 +44,8 @@ struct _EmpathySearchBarPriv
 
   GtkWidget *search_match_case;
 
+  GtkWidget *search_match_case_toolitem;
+
   GtkWidget *search_close;
   GtkWidget *search_previous;
   GtkWidget *search_next;
@@ -57,45 +62,6 @@ empathy_search_bar_new (EmpathyChatView *view)
   return GTK_WIDGET (self);
 }
 
-static void
-empathy_search_bar_get_preferred_height (GtkWidget *widget,
-    gint *minimun_height,
-    gint *natural_height)
-{
-  GtkBin *bin;
-  GtkWidget *child;
-
-  bin = GTK_BIN (widget);
-  child = gtk_bin_get_child (bin);
-
-  if (child && gtk_widget_get_visible (child))
-      gtk_widget_get_preferred_height (child, minimun_height, natural_height);
-}
-
-static void
-empathy_search_bar_size_allocate (GtkWidget *widget,
-    GtkAllocation *allocation)
-{
-  GtkBin *bin;
-  GtkWidget *child;
-  GtkAllocation child_allocation;
-
-  bin = GTK_BIN (widget);
-  child = gtk_bin_get_child (bin);
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  if (child && gtk_widget_get_visible (child))
-    {
-      child_allocation.x = allocation->x;
-      child_allocation.y = allocation->y;
-      child_allocation.width = MAX (allocation->width, 0);
-      child_allocation.height = MAX (allocation->height, 0);
-
-      gtk_widget_size_allocate (child, &child_allocation);
-    }
-}
-
 static void
 empathy_search_bar_update_buttons (EmpathySearchBar *self,
     gchar *search,
@@ -253,6 +219,42 @@ empathy_search_bar_match_case_toggled (GtkButton *button,
   empathy_search_bar_update (EMPATHY_SEARCH_BAR (user_data));
 }
 
+static void
+empathy_search_bar_match_case_menu_toggled (GtkWidget *check,
+    gpointer user_data)
+{
+  EmpathySearchBarPriv* priv = GET_PRIV ( EMPATHY_SEARCH_BAR (user_data));
+  gboolean match_case;
+
+  match_case = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (check));
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search_match_case),
+      match_case);
+}
+
+static gboolean
+empathy_searchbar_create_menu_proxy_cb (GtkToolItem *toolitem,
+    gpointer user_data)
+{
+  EmpathySearchBarPriv* priv = GET_PRIV ( EMPATHY_SEARCH_BAR (user_data));
+  GtkWidget *checkbox_menu;
+  gboolean match_case;
+
+  checkbox_menu = gtk_check_menu_item_new_with_mnemonic (_("_Match case"));
+  match_case = gtk_toggle_button_get_active (
+      GTK_TOGGLE_BUTTON (priv->search_match_case));
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (checkbox_menu),
+      match_case);
+
+  g_signal_connect (checkbox_menu, "toggled",
+      G_CALLBACK (empathy_search_bar_match_case_menu_toggled), user_data);
+
+  gtk_tool_item_set_proxy_menu_item (toolitem, "menu-proxy",
+      checkbox_menu);
+
+  return TRUE;
+}
+
 static void
 empathy_search_bar_init (EmpathySearchBar * self)
 {
@@ -285,12 +287,13 @@ empathy_search_bar_init (EmpathySearchBar * self)
       "search_previous", "clicked", empathy_search_bar_previous_cb,
       "search_next", "clicked", empathy_search_bar_next_cb,
       "search_match_case", "toggled", empathy_search_bar_match_case_toggled,
+      "search_match_case_toolitem", "create-menu-proxy", empathy_searchbar_create_menu_proxy_cb,
       NULL);
 
   g_signal_connect (G_OBJECT (self), "key-press-event",
       G_CALLBACK (empathy_search_bar_key_pressed), NULL);
 
-  gtk_container_add (GTK_CONTAINER (self), internal);
+  gtk_box_pack_start (GTK_BOX (self), internal, TRUE, TRUE, 0);
   gtk_widget_show_all (internal);
   gtk_widget_hide (priv->search_not_found);
   g_object_unref (gui);
@@ -300,13 +303,8 @@ static void
 empathy_search_bar_class_init (EmpathySearchBarClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 
   g_type_class_add_private (gobject_class, sizeof (EmpathySearchBarPriv));
-
-  /* Neither GtkBin nor GtkContainer seems to do this for us :( */
-  widget_class->get_preferred_height = empathy_search_bar_get_preferred_height;
-  widget_class->size_allocate = empathy_search_bar_size_allocate;
 }
 
 void
